from typing import List, Union, Any

from langchain_core.documents import Document
from langchain_core.embeddings import Embeddings

from aa_rag import setting
from aa_rag import utils
from aa_rag.gtypes import IndexType
from aa_rag.gtypes.enums import VectorDBType


class BaseIndex:
    _index_type: IndexType
    _indexed_data: Any

    def __init__(
        self,
        knowledge_name: str,
        vector_db: VectorDBType = VectorDBType.LANCE,
        embedding_model: str = setting.embedding.model,
        **kwargs,
    ):
        self._table_name = f"{knowledge_name}_{self.index_type}_{embedding_model}"

        self._vector_db = utils.get_vector_db(vector_db)
        self._embeddings, dimensions = utils.get_embedding_model(
            embedding_model, return_dim=True
        )

        if self.table_name not in self.vector_db.table_list():
            # create table if not exist
            if kwargs.get("schema"):
                self._vector_db.create_table(
                    self.table_name, schema=kwargs.get("schema")
                )
            else:
                import pyarrow as pa

                schema = pa.schema(
                    [
                        pa.field("id", pa.utf8(), False),
                        pa.field("vector", pa.list_(pa.float64(), dimensions), False),
                        pa.field("text", pa.utf8(), False),
                        pa.field(
                            "metadata",
                            pa.struct(
                                [
                                    pa.field("source", pa.utf8(), False),
                                ]
                            ),
                            False,
                        ),
                    ]
                )

                self._vector_db.create_table(self.table_name, schema=schema)
        else:
            pass

    @property
    def indexed_data(self):
        return self._indexed_data

    @property
    def index_type(self):
        return self._index_type

    @property
    def table_name(self):
        return self._table_name

    @property
    def vector_db(self):
        return self._vector_db

    @property
    def embeddings(self) -> Embeddings:
        return self._embeddings

    def index(self, source_docs: Union[Document | List[Document]]):
        """
        Index documents. Assign the return value to self.indexed_data.

        Args:
            source_docs (Union[Document  |  List[Document]]): Document instance or more base on langchain.
        """
        return NotImplemented

    def store(self, **kwargs):
        """
        Write self.indexed_data to the database.

        Args:
            **kwargs:
        """
        return NotImplemented

    def __repr__(self):
        return f"{self.__class__.__name__}({self.table_name})"



import hashlib
import uuid
from pathlib import Path

from langchain_core.documents import Document
from langchain_core.embeddings import Embeddings
from langchain_core.language_models import BaseChatModel
from langchain_openai import OpenAIEmbeddings, ChatOpenAI
from markitdown import MarkItDown

from aa_rag import setting
from aa_rag.db import LanceDBDataBase
from aa_rag.db.base import BaseVectorDataBase, BaseNoSQLDataBase
from aa_rag.db.tinydb_ import TinyDBDataBase
from aa_rag.gtypes.enums import VectorDBType, NoSQLDBType


def parse_file(file_path: Path) -> Document:
    """
    Parse a file and return a Document object.

    Args:
        file_path (str): Path to the file to be parsed.

    Returns:
        Document: Document object containing the parsed content and metadata.
    """
    assert file_path.exists(), f"File not found: {file_path}"

    md = MarkItDown()
    content_str = md.convert(str(file_path.absolute())).text_content

    return Document(page_content=content_str, metadata={"source": file_path.name})


def calculate_md5(input_string: str) -> str:
    """
    Calculate the MD5 hash of a string.

    Args:
        input_string (str): need to be calculated.

    Returns:
        str: MD5 hash of the input string.
    """
    md5_hash = hashlib.md5()
    md5_hash.update(input_string.encode("utf-8"))
    return md5_hash.hexdigest()


def get_embedding_model(
    model_name: str, return_dim: bool = False
) -> Embeddings | tuple[Embeddings, int]:
    """
    Get the embedding model based on the model name.
    Args:
        model_name (str): Model name.
        return_dim (bool): Return the embedding dimension if True.

    Returns:
        Embeddings: Embedding model instance.
        If return_dim is True, also returns the number of dimensions.

    """
    assert setting.openai.api_key, (
        "OpenAI API key is required for using OpenAI embeddings."
    )
    embeddings = OpenAIEmbeddings(
        model=model_name,
        dimensions=1536,
        api_key=setting.openai.api_key,
        base_url=setting.openai.base_url,
    )
    if return_dim:
        return embeddings, embeddings.dimensions or 1536
    else:
        return embeddings


def get_llm(model_name: str) -> BaseChatModel:
    assert setting.openai.api_key, (
        "OpenAI API key is required for using OpenAI embeddings."
    )
    model = ChatOpenAI(
        model=model_name,
        api_key=setting.openai.api_key,
        base_url=setting.openai.base_url,
    )

    return model


def get_vector_db(db_type: VectorDBType) -> BaseVectorDataBase | None:
    match db_type:
        case VectorDBType.LANCE:
            return LanceDBDataBase()
        case VectorDBType.MILVUS:
            pass
        case _:
            raise ValueError(f"Invalid db type: {db_type}")


def get_nosql_db(db_type: NoSQLDBType) -> BaseNoSQLDataBase | None:
    match db_type:
        case NoSQLDBType.TINYDB:
            return TinyDBDataBase()
        case NoSQLDBType.MONGODB:
            pass
        case _:
            raise ValueError(f"Invalid db type: {db_type}")


def get_db(
    db_type: NoSQLDBType | VectorDBType,
) -> BaseNoSQLDataBase | BaseVectorDataBase | None:
    if isinstance(db_type, NoSQLDBType):
        return get_nosql_db(db_type)
    elif isinstance(db_type, VectorDBType):
        return get_vector_db(db_type)
    else:
        raise ValueError(f"Invalid db type: {db_type}")


def get_uuid():
    return str(uuid.uuid4()).replace("-", "")



from pydantic import BaseModel, Field

from aa_rag import setting


class BaseKnowledgeItem(BaseModel):
    llm: str = Field(
        default=setting.llm.model,
        description="The language model used for the knowledge base",
    )
    embedding_model: str = Field(
        default=setting.embedding.model,
        description="The embedding model used for the knowledge base",
    )



from typing import List, Dict

from langchain_community.retrievers import BM25Retriever
from langchain_core.documents import Document

from aa_rag import setting
from aa_rag.gtypes.enums import RetrieveType, IndexType
from aa_rag.retrieve.base import BaseRetrieve


class BM25Retrieve(BaseRetrieve):
    _retrieve_type = RetrieveType.BM25

    def __init__(self, knowledge_name: str, index_type: IndexType, **kwargs):
        super().__init__(knowledge_name, index_type, **kwargs)

    def retrieve(
        self,
        query: str,
        top_k: int = setting.retrieve.k,
        only_page_content: bool = setting.retrieve.only_page_content,
        **kwargs,
    ) -> List[Dict | str]:
        """
        Retrieve documents by BM25.

        Args:
            query (str): Query string.
            top_k (int, optional): Number of documents to retrieve. Defaults to 3.
            only_page_content (bool, optional): Return only page content. Defaults to False.

        Returns:
            List[Dict|str]: List of retrieved documents.
        """

        with self.vector_db.get_table(self.table_name) as table:
            all_doc_df = table.select(where="1=1")
            all_doc_s = (
                all_doc_df[["text", "metadata"]]
                .apply(
                    lambda x: Document(page_content=x["text"], metadata=x["metadata"]),
                    axis=1,
                )
                .tolist()
            )
        sparse_retriever = BM25Retriever.from_documents(all_doc_s)
        sparse_retriever.k = top_k

        result: List[Document] = sparse_retriever.invoke(query)

        if only_page_content:
            return [doc.page_content for doc in result]
        else:
            return [doc.model_dump(exclude={"id", "type"}) for doc in result]



from typing import Any, List

from pandas import DataFrame


class BaseDataBase:
    _db_type: str
    _conn_obj: Any
    _table_obj: Any

    def __init__(self, **kwargs):
        pass

    @property
    def connection(self):
        return self._conn_obj

    @property
    def table(self):
        return self._table_obj

    def connect(self):
        return NotImplemented

    def get_table(self, table_name, **kwargs):
        return NotImplemented

    def table_list(self) -> List[str]:
        return NotImplemented

    def create_table(self, table_name, schema, **kwargs):
        return NotImplemented

    def drop_table(self, table_name):
        return NotImplemented

    def insert(self, **kwargs):
        return NotImplemented

    def select(self, **kwargs):
        return NotImplemented

    def update(self, **kwargs):
        return NotImplemented

    def delete(self, **kwargs):
        return NotImplemented

    def close(self):
        return NotImplemented

    def __enter__(self):
        assert self.table is not None, (
            "Table object is not defined, please use get_table() method to define it"
        )

        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self._table_obj = None  # Reset the table object

        return False


class BaseVectorDataBase(BaseDataBase):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def insert(self, data: list[dict] | DataFrame, **kwargs):
        return NotImplemented

    def update(self, where: str, values: dict, **kwargs):
        return NotImplemented

    def delete(self, where: str):
        return NotImplemented

    def upsert(self, data: list[dict] | DataFrame, duplicate_where, **kwargs):
        assert self.table is not None, (
            "Table object is not defined, please use `with db.get_table()` to use insert method"
        )

        self.delete(duplicate_where)
        self.insert(data, **kwargs)

    def overwrite(self, data: list[dict] | DataFrame, **kwargs):
        assert self.table is not None, (
            "Table object is not defined, please use `with db.get_table()` to use overwrite method"
        )

        self.delete(where="1=1")
        self.insert(data, **kwargs)

    def search(self, query_vector: List[float], top_k: int = 3, **kwargs):
        return NotImplemented


class BaseNoSQLDataBase(BaseDataBase):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def create_table(self, table_name, **kwargs):
        return NotImplemented



from pydantic import BaseModel, Field, ConfigDict, FilePath

from aa_rag import setting
from aa_rag.gtypes import IndexType
from aa_rag.gtypes.enums import DBMode
from aa_rag.gtypes.models.base import BaseResponse


class IndexItem(BaseModel):
    knowledge_name: str = Field(default=..., examples=["fairy_tale"])
    index_type: IndexType = Field(
        default=setting.index.type, examples=[setting.index.type]
    )
    embedding_model: str = Field(
        default=setting.embedding.model, examples=[setting.embedding.model]
    )

    model_config = ConfigDict(extra="allow")


class ChunkIndexItem(IndexItem):
    file_path: FilePath = Field(default=..., examples=["./data/fairy_tale.txt"])
    db_mode: DBMode = Field(
        default=setting.db.vector.mode,
        examples=[setting.db.vector.mode],
        description="Mode for inserting data to db",
    )
    chunk_size: int = Field(
        default=setting.index.chunk_size, examples=[setting.index.chunk_size]
    )
    chunk_overlap: int = Field(
        default=setting.index.overlap_size, examples=[setting.index.overlap_size]
    )
    index_type: IndexType = Field(
        default=setting.index.type, examples=[setting.index.type]
    )

    model_config = ConfigDict(extra="forbid")


class IndexResponse(BaseResponse):
    class Data(BaseModel):
        table_name: str = Field(..., examples=["fairy_tale_chunk_text_embedding_model"])

    message: str = Field(
        default="Indexing completed via ChunkIndex",
        examples=["Indexing completed via ChunkIndex"],
    )
    data: Data = Field(default_factory=Data)



from pathlib import Path

from tinydb import TinyDB, Query
from tinydb.table import Table

from aa_rag import setting
from aa_rag.db.base import BaseNoSQLDataBase


class TinyDBDataBase(BaseNoSQLDataBase):
    _db_type = "TinyDB"

    def __init__(self, uri: str = setting.db.nosql.uri, **kwargs):
        self.uri = uri
        # 如果目录不存在，则创建父目录
        Path(self.uri).parent.mkdir(parents=True, exist_ok=True)

        self._conn_obj = self.connect()
        self._table_obj = None

        super().__init__(**kwargs)

    @property
    def connection(self) -> TinyDB:
        return self._conn_obj

    @property
    def table(self) -> Table:
        if self._table_obj is None:
            raise ValueError(
                "Table 对象未定义，请先调用 `get_table(table_name)` 方法设置 table 对象"
            )
        return self._table_obj

    def connect(self):
        return TinyDB(self.uri)

    def create_table(self, table_name, **kwargs):
        """创建并返回指定名称的表"""
        return self.connection.table(table_name, **kwargs)

    def get_table(self, table_name, **kwargs):
        """设置当前使用的表，并返回 self 以支持链式调用"""
        self._table_obj = self.connection.table(table_name, **kwargs)
        return self

    def table_list(self):
        """返回所有表的名称集合"""
        return self.connection.tables()

    def drop_table(self, table_name):
        """删除指定名称的表"""
        return self.connection.drop_table(table_name)

    def insert(self, data):
        """
        插入数据。
        注意：此处 data 可以为单个 dict 或 dict 列表，根据 tinydb 的 insert/insert_multiple 方法自行扩展。
        """
        return self.table.insert(data)

    def _build_query_mongo(self, mongo_query: dict, q: Query = None):
        """
        将 MongoDB 查询语法转换为 TinyDB 的查询表达式。

        支持的 MongoDB 查询操作符包括：
          - 比较操作符: $gt, $gte, $lt, $lte, $eq, $ne
          - 集合操作符: $in, $nin
          - 存在性判断: $exists
          - 逻辑操作符: $and, $or, $nor, $not

        示例：
          {"age": {"$gt": 30, "$lt": 50}, "name": "John"}
          等价于：
          (Query()['age'] > 30) & (Query()['age'] < 50) & (Query()['name'] == "John")
        """
        if q is None:
            q = Query()
        if isinstance(mongo_query, dict):
            # 处理顶层逻辑操作符
            if "$and" in mongo_query:
                sub_exprs = [
                    self._build_query_mongo(sub, q) for sub in mongo_query["$and"]
                ]
                expr = sub_exprs[0]
                for e in sub_exprs[1:]:
                    expr = expr & e
                return expr
            elif "$or" in mongo_query:
                sub_exprs = [
                    self._build_query_mongo(sub, q) for sub in mongo_query["$or"]
                ]
                expr = sub_exprs[0]
                for e in sub_exprs[1:]:
                    expr = expr | e
                return expr
            elif "$nor" in mongo_query:
                sub_exprs = [
                    self._build_query_mongo(sub, q) for sub in mongo_query["$nor"]
                ]
                expr = sub_exprs[0]
                for e in sub_exprs[1:]:
                    expr = expr | e
                return ~expr
            elif "$not" in mongo_query:
                sub_expr = self._build_query_mongo(mongo_query["$not"], q)
                return ~sub_expr
            else:
                # 处理字段查询
                sub_exprs = []
                for field, condition in mongo_query.items():
                    if isinstance(condition, dict):
                        field_expr = None
                        for op, val in condition.items():
                            if op == "$gt":
                                current = q[field] > val
                            elif op == "$gte":
                                current = q[field] >= val
                            elif op == "$lt":
                                current = q[field] < val
                            elif op == "$lte":
                                current = q[field] <= val
                            elif op == "$eq":
                                current = q[field] == val
                            elif op == "$ne":
                                current = ~(q[field] == val)
                            elif op == "$in":
                                if not isinstance(val, list):
                                    raise ValueError("$in 操作符要求值为列表")
                                # 使用 test 方法构造 in 查询
                                current = q[field].test(lambda v, lst=val: v in lst)
                            elif op == "$nin":
                                if not isinstance(val, list):
                                    raise ValueError("$nin 操作符要求值为列表")
                                current = ~(q[field].test(lambda v, lst=val: v in lst))
                            elif op == "$exists":
                                # 若 val 为 True，则判断字段值不为 None；若为 False，则判断为 None
                                current = (
                                    (q[field] is not None)
                                    if val
                                    else (q[field] is None)
                                )
                            else:
                                raise ValueError(f"不支持的操作符: {op}")
                            if field_expr is None:
                                field_expr = current
                            else:
                                field_expr = field_expr & current
                        sub_exprs.append(field_expr)
                    else:
                        # 直接进行等值判断
                        sub_exprs.append(q[field] == condition)
                if not sub_exprs:
                    return None
                expr = sub_exprs[0]
                for e in sub_exprs[1:]:
                    expr = expr & e
                return expr
        else:
            raise ValueError("Mongo 查询条件必须为字典类型")

    def select(self, query: dict = None):
        """
        查询接口，接收 MongoDB 查询语法。

        示例：
            query = {"age": {"$gt": 30}, "$or": [{"name": "Alice"}, {"name": "Bob"}]}
            results = db.select(query)
        """
        if query is None or not query:
            return self.table.all()
        query_obj = self._build_query_mongo(query)
        return self.table.search(query_obj)

    def update(self, update_data: dict, query: dict = None):
        """
        更新符合 MongoDB 查询条件的记录。

        Args:
            update_data: 一个字典，指定要更新的字段和值。
            query: MongoDB 查询语法的字典，用于选择需要更新的记录。
                   如果 query 为 None 或空字典，则更新所有记录。

        Returns:
            update_result: 通常为更新的记录标识列表。
        """
        if query is None or not query:
            # 若没有指定查询条件，则更新所有记录
            return self.table.update(update_data)
        query_obj = self._build_query_mongo(query)
        return self.table.update(update_data, query_obj)

    def delete(self, query: dict = None):
        """
        删除符合 MongoDB 查询条件的记录。

        Args:
            query: MongoDB 查询语法的字典，用于选择需要删除的记录。
                   如果 query 为 None 或空字典，则删除所有记录。

        Returns:
            delete_result: 通常为删除的记录标识列表。
        """
        if query is None or not query:
            # 若没有指定查询条件，则删除所有记录
            return self.table.remove()
        query_obj = self._build_query_mongo(query)
        return self.table.remove(query_obj)



from pydantic import BaseModel, Field, ConfigDict

from aa_rag import setting
from aa_rag.gtypes import IndexType
from aa_rag.gtypes.enums import RetrieveType
from aa_rag.gtypes.models.base import BaseResponse


class RetrieveItem(BaseModel):
    knowledge_name: str = Field(default=..., examples=["fairy_tale"])
    index_type: IndexType = Field(
        default=setting.index.type, examples=[setting.index.type]
    )
    retrieve_type: RetrieveType = Field(
        default=setting.retrieve.type, examples=[setting.retrieve.type]
    )
    embedding_model: str = Field(
        default=setting.embedding.model, examples=[setting.embedding.model]
    )

    top_k: int = Field(default=setting.retrieve.type, examples=[setting.retrieve.type])
    only_page_content: bool = Field(
        default=setting.retrieve.only_page_content,
        examples=[setting.retrieve.only_page_content],
    )

    model_config = ConfigDict(extra="allow")


class HybridRetrieveItem(RetrieveItem):
    query: str = Field(default=..., examples=["What is the story of Cinderella?"])
    weight_dense: float = Field(
        default=setting.retrieve.weight.dense,
        examples=[setting.retrieve.weight.dense],
    )
    weight_sparse: float = Field(
        default=setting.retrieve.weight.sparse,
        examples=[setting.retrieve.weight.sparse],
    )

    model_config = ConfigDict(extra="forbid")


class DenseRetrieveItem(RetrieveItem):
    query: str = Field(default=..., examples=["What is the story of Cinderella?"])

    model_config = ConfigDict(extra="forbid")


class BM25RetrieveItem(RetrieveItem):
    query: str = Field(default=..., examples=["What is the story of Cinderella?"])

    model_config = ConfigDict(extra="forbid")


class RetrieveResponse(BaseResponse):
    class Data(BaseModel):
        documents: list = Field(default=..., examples=[[]])

    message: str = Field(
        default="Retrieval completed via BaseRetrieve", examples=["Retrieval completed"]
    )
    data: Data = Field(default_factory=Data)



from fastapi import APIRouter, HTTPException

from aa_rag import utils
from aa_rag.gtypes import IndexType
from aa_rag.gtypes.models.index import IndexItem, ChunkIndexItem, IndexResponse
from aa_rag.index.chunk import ChunkIndex

router = APIRouter(
    prefix="/index", tags=["Index"], responses={404: {"description": "Not found"}}
)


@router.post("/")
async def root(item: IndexItem):
    match item.index_type:
        case IndexType.CHUNK:
            chunk_item = ChunkIndexItem(**item.model_dump())
            return await chunk_index(chunk_item)
        case _:
            raise HTTPException(status_code=400, detail="IndexType not supported")


@router.post("/chunk")
async def chunk_index(item: ChunkIndexItem) -> IndexResponse:
    indexer = ChunkIndex(**item.model_dump(exclude={"file_path", "mode"}))
    source_docs = utils.parse_file(item.file_path)

    indexer.index(source_docs)
    indexer.store(mode=item.db_mode)
    return IndexResponse(
        code=200,
        status="success",
        message="Indexing completed via ChunkIndex",
        data=IndexResponse.Data(
            table_name=indexer.table_name,
        ),
    )



from typing import Any, Optional, List

from pydantic import BaseModel, Field, AnyUrl, ConfigDict

from aa_rag.gtypes.models.base import BaseResponse
from aa_rag.gtypes.models.knowlege_base.base import BaseKnowledgeItem


class CompatibleEnv(BaseModel):
    # Platform and Operating System (required)
    platform: Any = Field(
        ..., description="The operating system platform, e.g., Darwin 24.1.0"
    )
    arch: Any = Field(..., description="The system architecture, e.g., arm64")

    model_config = ConfigDict(extra="allow")


class Guide(BaseModel):
    procedure: str = Field(
        ..., description="The detailed deployment process of the guide"
    )
    compatible_env: CompatibleEnv = Field(
        ..., description="The compatible environment of the guide"
    )


class Project(BaseModel):
    name: str = Field(..., description="Project name")
    id: Optional[str] = Field(None, description="Project ID")
    description: Optional[str] = Field(None, description="Project description")
    git_url: Optional[AnyUrl] = Field(None, description="Git URL of the project")
    guides: Optional[List[Guide]] = Field(
        None, description="The deployment guide of the project"
    )


class SolutionIndexItem(BaseKnowledgeItem):
    env_info: CompatibleEnv = Field(..., description="The environment information")
    procedure: str = Field(..., description="The deployment procedure of the solution")

    class Project_Meta(BaseModel):
        name: str = Field(..., description="Project name")
        model_config = ConfigDict(extra="allow")

    project_meta: Project_Meta = Field(..., description="The project meta information")


class SolutionRetrieveItem(BaseKnowledgeItem):
    env_info: CompatibleEnv = Field(..., description="The environment information")
    project_meta: SolutionIndexItem.Project_Meta = Field(
        ..., description="The project meta information"
    )


class SolutionIndexResponse(BaseResponse):
    class Data(BaseModel):
        affect_row_num: int = Field(default=..., examples=[0])
        table_name: str = Field("solution", examples=["solution"])

    message: str = Field(
        default="Indexing completed in Solution Knowledge Base",
        examples=["Indexing completed in Solution Knowledge Base"],
    )
    data: Data = Field(default_factory=Data)


class SolutionRetrieveResponse(BaseResponse):
    class Data(BaseModel):
        guide: Guide | None = Field(
            default=...,
            examples=[
                Guide(
                    procedure="xxx",
                    compatible_env=CompatibleEnv(platform="xxx", arch="xxx"),
                )
            ],
        )

    message: str = Field(
        default="Retrieval completed in Solution Knowledge Base",
        examples=["Retrieval completed in Solution Knowledge Base"],
    )
    data: Data = Field(default_factory=Data)



from aa_rag import utils, setting
from aa_rag.db.base import BaseVectorDataBase
from aa_rag.gtypes.enums import RetrieveType, IndexType, VectorDBType


class BaseRetrieve:
    _retrieve_type: RetrieveType

    def __init__(
        self,
        knowledge_name: str,
        index_type: IndexType,
        vector_db: VectorDBType = VectorDBType.LANCE,
        embedding_model: str = setting.embedding.model,
        **kwargs,
    ):
        self._table_name = f"{knowledge_name}_{index_type}_{embedding_model}"
        self._vector_db = utils.get_vector_db(vector_db)
        self._embeddings = utils.get_embedding_model(embedding_model, return_dim=False)

        assert self.table_name in self.vector_db.table_list(), (
            f"Table {self.table_name} not found in {self.vector_db}"
        )

    @property
    def table_name(self):
        return self._table_name

    @property
    def vector_db(self) -> BaseVectorDataBase:
        return self._vector_db

    @property
    def embeddings(self):
        return self._embeddings

    def retrieve(
        self,
        query: str,
        top_k: int = 3,
        only_page_content: bool = False,
        **kwargs,
    ):
        return NotImplementedError



from typing import Any

from fastapi import status
from pydantic import BaseModel, Field


class BaseResponse(BaseModel):
    code: int = Field(..., examples=[status.HTTP_200_OK])
    status: str = Field(default=..., examples=["success"])
    message: str = Field(default=...)
    data: Any = Field(default=...)



from typing import List

import lancedb
from lancedb import DBConnection
from lancedb.pydantic import LanceModel
from lancedb.table import Table
from pandas import DataFrame

from aa_rag import setting
from aa_rag.db.base import BaseVectorDataBase


class LanceDBDataBase(BaseVectorDataBase):
    _db_type = "LanceDB"

    def __init__(self, uri: str = setting.db.vector.uri, **kwargs):
        self.uri = uri
        self._conn_obj = self.connect()
        super().__init__(**kwargs)

    @property
    def connection(self) -> DBConnection:
        return self._conn_obj

    @property
    def table(self) -> Table:
        return self._table_obj

    def connect(self, **kwargs) -> DBConnection:
        return lancedb.connect(self.uri, **kwargs)

    def table_list(self, **kwargs) -> List[str]:
        table_name_s = self.connection.table_names(**kwargs)
        return list(table_name_s)

    def get_table(self, table_name, **kwargs):
        self._table_obj = self.connection.open_table(table_name, **kwargs)

        return self

    def create_table(self, table_name, schema: LanceModel, **kwargs):
        self.connection.create_table(name=table_name, schema=schema, **kwargs)

    def drop_table(self, table_name):
        return self.connection.drop_table(table_name)

    def select(self, where: str = None, **kwargs) -> DataFrame:
        assert self.table is not None, (
            "Table object is not defined, please use `with db.get_table()` to use select method"
        )

        return self.table.search().where(where).to_pandas()

    def insert(self, data: list[dict] | DataFrame, **kwargs):
        assert self.table is not None, (
            "Table object is not defined, please use `with db.get_table()` to use insert method"
        )

        self.table.add(data, **kwargs)

    def update(self, where: str, values: dict, **kwargs):
        assert self.table is not None, (
            "Table object is not defined, please use `with db.get_table()` to use update method"
        )

        self.table.update(where=where, values=values, **kwargs)

    def delete(self, where: str):
        assert self.table is not None, (
            "Table object is not defined, please use `with db.get_table()` to use delete method"
        )

        self.table.delete(where=where)

    def search(self, query_vector: List[float], top_k: int = 3, **kwargs):
        assert self.table is not None, (
            "Table object is not defined, please use `with db.get_table()` to use search method"
        )

        return self.table.search(query_vector).to_list()[:top_k]



from typing import List, Any

from langchain_core.documents import Document

from aa_rag.gtypes.enums import VectorDBType
from aa_rag.index.chunk import ChunkIndex
from aa_rag.knowledge_base.base import BaseKnowledge
from aa_rag.retrieve.hybrid import HybridRetrieve


class QAKnowledge(BaseKnowledge):
    _knowledge_name = "QA"

    def __init__(
        self,
        vector_db: VectorDBType = VectorDBType.LANCE,
        **kwargs,
    ):
        """
        QA Knowledge Base. Built-in Knowledge Base.
        """
        super().__init__(**kwargs)

        # define table schema
        import pyarrow as pa

        schema = pa.schema(
            [
                pa.field("id", pa.utf8(), False),
                pa.field("vector", pa.list_(pa.float64(), self.dimensions), False),
                pa.field("text", pa.utf8(), False),
                pa.field(
                    "metadata",
                    pa.struct(
                        [
                            pa.field("solution", pa.utf8(), False),
                            pa.field("tags", pa.list_(pa.utf8()), False),
                        ]
                    ),
                    False,
                ),
            ]
        )

        self._indexer = ChunkIndex(
            knowledge_name=self.knowledge_name.lower(),
            vector_db=vector_db,
            schema=schema,
        )
        self.db = vector_db

    def index(
        self, error_desc: str, error_solution: str, tags: List[str], **kwargs
    ) -> List[str]:
        """
        Index the QA information.
        Args:
            error_desc: The error description.
            error_solution: The solution of the QA.
            tags: The tags of the QA.
            **kwargs:
        Returns:
            List[str]: List of document id what be inserted.
        """
        # check if the project is already indexed

        self._indexer.chunk_size = (
            len(error_desc) * 2
            if kwargs.get("chunk_size") is None
            else kwargs.get("chunk_size")
        )
        self._indexer.chunk_overlap = (
            0 if kwargs.get("chunk_overlap") is None else kwargs.get("chunk_overlap")
        )

        self._indexer.index(
            Document(
                page_content=error_desc,
                metadata={"solution": error_solution, "tags": tags},
            )
        )

        return self._indexer.store()

    def retrieve(self, error_desc: str, tags: List[str] = None) -> List[Any]:
        """
        Retrieve the QA information.
        Args:
            error_desc: The error description.
            tags: The tags of the QA.
        Returns:
            List[Any]: The QA information.
        """
        retriever = HybridRetrieve(
            knowledge_name=self.knowledge_name.lower(),
            index_type=self._indexer.index_type,
            vector_db=self.db,
        )
        return retriever.retrieve(query=error_desc, top_k=1, only_page_content=False)



from typing import List, Dict

from langchain_community.vectorstores import LanceDB
from langchain_core.documents import Document

from aa_rag import setting
from aa_rag.gtypes.enums import RetrieveType, IndexType
from aa_rag.retrieve.base import BaseRetrieve


class DenseRetrieve(BaseRetrieve):
    _retrieve_type = RetrieveType.DENSE

    def __init__(self, knowledge_name: str, index_type: IndexType, **kwargs):
        super().__init__(knowledge_name, index_type, **kwargs)

    def retrieve(
        self,
        query: str,
        top_k: int = setting.retrieve.k,
        only_page_content: bool = setting.retrieve.only_page_content,
        **kwargs,
    ) -> List[Dict | str]:
        """
        Retrieve documents by dense.

        Args:
            query (str): Query string.
            top_k (int, optional): Number of documents to retrieve. Defaults to 3.
            only_page_content (bool, optional): Return only page content. Defaults to False.

        Returns:
            List[Dict|str]: List of retrieved documents.
        """

        # dense retrieval
        dense_retriever = LanceDB(
            connection=self.vector_db.connection,
            table_name=self.table_name,
            embedding=self.embeddings,
        )

        result: List[Document] = dense_retriever.similarity_search(query, k=top_k)

        if only_page_content:
            return [doc.page_content for doc in result]
        else:
            return [doc.model_dump(exclude={"id", "type"}) for doc in result]



from typing import List, Union

from langchain.text_splitter import RecursiveCharacterTextSplitter
from langchain_core.documents import Document

from aa_rag import setting
from aa_rag import utils
from aa_rag.gtypes import IndexType
from aa_rag.gtypes.enums import DBMode
from aa_rag.index.base import BaseIndex


class ChunkIndex(BaseIndex):
    _index_type = IndexType.CHUNK

    def __init__(
        self,
        knowledge_name: str,
        chunk_size=setting.index.chunk_size,
        chunk_overlap=setting.index.overlap_size,
        **kwargs,
    ):
        super().__init__(knowledge_name, **kwargs)

        self.chunk_size = chunk_size
        self.chunk_overlap = chunk_overlap

    def index(
        self,
        source_docs: Union[Document | List[Document]],
    ):
        if isinstance(source_docs, Document):
            source_docs = [source_docs]

        # split the document into chunks
        splitter = RecursiveCharacterTextSplitter(
            chunk_size=self.chunk_size, chunk_overlap=self.chunk_overlap
        )
        self._indexed_data = splitter.split_documents(source_docs)

    def store(self, mode=setting.db.vector.mode):
        """
        Insert documents to vector db.

        Args:
            mode (str, optional): Insert method. Defaults to 'deinsert'.

                - `insert`: Insert new documents to db directly without removing duplicate.
                - `upsert`: Insert new documents to db, if document existed, update it.

        """
        assert self.indexed_data, "Can not store because indexed data is empty."
        # assert self.db, "Can not store because db is empty."

        # detects whether the metadata has an id field. If not, it will be generated id based on page_content via md5 algorithm.
        id_s = [
            doc.metadata.get("id", utils.calculate_md5(doc.page_content))
            for doc in self.indexed_data
        ]

        text_vector_s = self.embeddings.embed_documents(
            [_.page_content for _ in self.indexed_data]
        )
        data = []

        for id_, vector, doc in zip(id_s, text_vector_s, self.indexed_data):
            data.append(
                {
                    "id": id_,
                    "vector": vector,
                    "text": doc.page_content,
                    "metadata": doc.metadata,
                }
            )

        match mode:
            case DBMode.INSERT:
                with self.vector_db.get_table(self.table_name) as table:
                    table.add(data)
                return id_s

            case DBMode.UPSERT:
                ids_str = ", ".join(map(lambda x: f"'{x}'", id_s))
                where_str = f"id IN ({ids_str})"

                with self.vector_db.get_table(self.table_name) as table:
                    table.upsert(data, duplicate_where=where_str)

            case DBMode.OVERWRITE:
                with self.vector_db.get_table(self.table_name) as table:
                    table.overwrite(data)

            case _:
                raise ValueError(f"Invalid mode: {mode}")



import json
from typing import Dict, Any, List

from langchain_core.output_parsers import StrOutputParser
from langchain_core.prompts import ChatPromptTemplate

from aa_rag import utils
from aa_rag.db.base import BaseNoSQLDataBase
from aa_rag.gtypes.enums import NoSQLDBType
from aa_rag.gtypes.models.knowlege_base.solution import CompatibleEnv, Project, Guide
from aa_rag.knowledge_base.base import BaseKnowledge


class SolutionKnowledge(BaseKnowledge):
    _knowledge_name = "Solution"

    def __init__(self, nosql_db: NoSQLDBType = NoSQLDBType.TINYDB, **kwargs):
        """
        Solution Knowledge Base. Built-in Knowledge Base.
        Args:
            nosql_db: 指定使用的 NoSQL 数据库类型（此处默认使用 TinyDB）
            **kwargs: 其他关键字参数
        """
        super().__init__(**kwargs)
        self.nosql_db: BaseNoSQLDataBase = utils.get_db(nosql_db)
        self.table_name = self.knowledge_name.lower()
        # 初始化 TinyDB 表（如果表不存在，则自动创建）
        self.nosql_db.get_table(self.table_name)

    def _is_compatible_env(
        self, source_env_info: CompatibleEnv, target_env_info: CompatibleEnv
    ) -> bool:
        """
        判断 source_env_info 与 target_env_info 是否兼容。
        """
        prompt_template = ChatPromptTemplate.from_messages(
            [
                (
                    "system",
                    """You are an expert in computer hardware device information. 
                    I will provide you with two jsons. Each json is the detailed data of a computer hardware device information.
                    --Requirements--
                    1. Please determine whether the two devices are compatible. If compatible, please return "True". Otherwise, return "False".
                    2. Do not return other information. Just return "True" or "False".
                    
                    --Data--
                    source_env_info: {source_env_info}
                    target_env_info: {target_env_info}
                    
                    --Result--
                    result:
                    """,
                )
            ]
        )

        chain = prompt_template | self.llm | StrOutputParser()
        result = chain.invoke(
            {
                "source_env_info": json.dumps(source_env_info.model_dump()),
                "target_env_info": json.dumps(target_env_info.model_dump()),
            }
        )
        try:
            result = bool(result)
        except Exception:
            result = False
        return result

    def _get_project_in_db(self, project_meta: Dict[str, Any]) -> Project | None:
        """
        从 TinyDB 中查询项目记录（通过项目名称进行匹配），并返回 Project 对象。
        """
        # 为方便查询，将项目名称在顶层存储，因此这里直接查询 "name" 字段
        query = {"name": project_meta["name"]}
        with self.nosql_db.get_table(self.table_name) as table:
            records = table.select(query)
        if records:
            record = records[0]
            # 从记录中还原 guides 列表
            guides_data: List[Dict[str, Any]] = record.get("guides", [])
            guides: List[Guide] = [
                Guide(
                    procedure=item["procedure"],
                    compatible_env=CompatibleEnv(**item["compatible_env"]),
                )
                for item in guides_data
            ]
            # 获取记录中存储的 project_id
            project_id = record.get("project_id", None)
            return Project(
                **record.get("project_meta", {}), guides=guides, id=project_id
            )
        else:
            return None

    def _project_to_db(self, project: Project) -> int:
        """
        将项目保存到 TinyDB 中：
          - 若 project.id 为 None，则执行插入操作，并利用返回的 doc_id 作为项目 id；
          - 否则执行更新操作。
        Returns:
            1（表示操作成功）
        """
        # 准备待保存的数据：
        # 这里除了存储 guides 和 project_meta 外，还将 project_meta 中的 "name" 字段在顶层存储，便于查询。
        record = {
            "guides": [guide.model_dump() for guide in project.guides],
            "project_meta": project.model_dump(exclude={"guides", "id"}),
            "name": project.model_dump(exclude={"guides", "id"}).get("name"),
        }
        table = self.nosql_db.get_table(self.table_name)
        if project.id is None:
            # generate project id
            project_id = utils.get_uuid()
            record["project_id"] = project_id
            table.insert(record)
        else:
            # 更新指定 doc_id 的记录
            table.update(record, query={"project_id": project.id})
        return 1  # 此处返回 1 表示成功

    def _merge_procedure(self, source_procedure: str, target_procedure: str) -> str:
        """
        合并 source_procedure 与 target_procedure，返回合并后的 MarkDown 格式流程。
        """
        prompt_template = ChatPromptTemplate.from_messages(
            [
                (
                    "system",
                    """Merge the source procedure with the target procedure.
                    --Requirements--
                    1. The merged procedure should be in a MarkDown format.
                    2. Just return the merged procedure. Do not return other information.
                    --Data--
                    source_procedure: {source_procedure}
                    target_procedure: {target_procedure}
                    --Result--
                    merged_procedure:
                    """,
                )
            ]
        )

        chain = prompt_template | self.llm | StrOutputParser()
        result: str = chain.invoke(
            {"source_procedure": source_procedure, "target_procedure": target_procedure}
        )

        return result

    def index(
        self, env_info: Dict[str, Any], procedure: str, project_meta: Dict[str, Any]
    ) -> int:
        """
        将解决方案写入知识库：
          - 若已有相同项目，则判断环境兼容性，兼容则合并流程，不兼容则新增 guide；
          - 若没有相同项目，则新建项目记录。
        Returns:
            1（表示操作成功）
        """
        env_info_obj = CompatibleEnv(**env_info)

        project = self._get_project_in_db(project_meta)
        if project:
            for guide in project.guides:
                is_compatible: bool = self._is_compatible_env(
                    env_info_obj, guide.compatible_env
                )
                if is_compatible:
                    # 合并流程
                    merged_procedure = self._merge_procedure(guide.procedure, procedure)
                    guide.procedure = merged_procedure
                    break
            else:  # 如果没有环境兼容的 guide，则新增一个 guide
                guide = Guide(procedure=procedure, compatible_env=env_info_obj)
                project.guides.append(guide)
        else:  # 新建项目
            guide = Guide(procedure=procedure, compatible_env=env_info_obj)
            project = Project(guides=[guide], **project_meta)

        # 将项目数据保存到 TinyDB
        return self._project_to_db(project)

    def retrieve(
        self, env_info: Dict[str, Any], project_meta: Dict[str, Any]
    ) -> Guide | None:
        """
        从知识库中检索与给定环境兼容的 guide。
        """
        env_info_obj = CompatibleEnv(**env_info)
        project = self._get_project_in_db(project_meta)
        if project:
            for guide in project.guides:
                is_compatible: bool = self._is_compatible_env(
                    env_info_obj, guide.compatible_env
                )
                if is_compatible:
                    return guide
        return None



from typing import List, Optional

from pydantic import BaseModel, Field

from aa_rag.gtypes.models.base import BaseResponse


class QAIndexItem(BaseModel):
    error_desc: str = Field(
        ..., examples=["error_desc"], description="The error description"
    )
    error_solution: str = Field(
        ..., examples=["error_solution"], description="The error solution"
    )
    tags: list[str] = Field(..., examples=[["tags"]], description="The tags of the QA")


class QAIndexResponse(BaseResponse):
    class Data(BaseModel):
        affect_row_ids: Optional[List[str]] = Field(
            None, examples=["1"], description="The id of the inserted row"
        )

    data: Data = Field(..., description="The data of the response")


class QARetrieveItem(BaseModel):
    error_desc: str = Field(
        ..., examples=["error_desc"], description="The error description"
    )
    tags: list[str] | None = Field(
        None, examples=[["tags"]], description="The tags of the QA"
    )


class QARetrieveResponse(BaseResponse):
    class Data(BaseModel):
        qa: List[dict] = Field(
            ...,
            examples=[
                {
                    "error_desc": "error_desc",
                    "error_solution": "error_solution",
                    "tags": ["tags"],
                }
            ],
        )

    data: Data = Field(..., description="The data of the response")



from fastapi import APIRouter, HTTPException

from aa_rag.gtypes.enums import RetrieveType
from aa_rag.gtypes.models.retrieve import (
    RetrieveItem,
    HybridRetrieveItem,
    DenseRetrieveItem,
    BM25RetrieveItem,
    RetrieveResponse,
)
from aa_rag.retrieve.bm25 import BM25Retrieve
from aa_rag.retrieve.dense import DenseRetrieve
from aa_rag.retrieve.hybrid import HybridRetrieve

router = APIRouter(
    prefix="/retrieve", tags=["Retrieve"], responses={404: {"description": "Not found"}}
)


@router.post("/")
async def root(item: RetrieveItem):
    match item.retrieve_type:
        case RetrieveType.HYBRID:
            hybrid_item = HybridRetrieveItem(**item.model_dump())
            return await hybrid_retrieve(hybrid_item)
        case RetrieveType.DENSE:
            dense_item = DenseRetrieveItem(**item.model_dump())
            return await dense_retrieve(dense_item)
        case RetrieveType.BM25:
            bm25_item = BM25RetrieveItem(**item.model_dump())
            return await bm25_retrieve(bm25_item)
        case _:
            raise HTTPException(status_code=400, detail="RetrieveType not supported")


@router.post("/hybrid")
async def hybrid_retrieve(item: HybridRetrieveItem) -> RetrieveResponse:
    retriever = HybridRetrieve(**item.model_dump())

    result = retriever.retrieve(
        query=item.query,
        top_k=item.top_k,
        only_page_content=item.only_page_content,
        dense_weight=item.weight_dense,
        sparse_weight=item.weight_sparse,
    )

    return RetrieveResponse(
        code=200,
        status="success",
        message="Retrieval completed via HybridRetrieve",
        data=RetrieveResponse.Data(documents=result),
    )


@router.post("/dense")
async def dense_retrieve(item: DenseRetrieveItem) -> RetrieveResponse:
    retriever = DenseRetrieve(**item.model_dump())

    result = retriever.retrieve(
        query=item.query, top_k=item.top_k, only_page_content=item.only_page_content
    )

    return RetrieveResponse(
        code=200,
        status="success",
        message="Retrieval completed via DenseRetrieve",
        data=RetrieveResponse.Data(documents=result),
    )


@router.post("/bm25")
async def bm25_retrieve(item: BM25RetrieveItem) -> RetrieveResponse:
    retriever = BM25Retrieve(**item.model_dump())

    result = retriever.retrieve(
        query=item.query, top_k=item.top_k, only_page_content=item.only_page_content
    )

    return RetrieveResponse(
        code=200,
        status="success",
        message="Retrieval completed via BM25Retrieve",
        data=RetrieveResponse.Data(documents=result),
    )



import ast
import os
from pathlib import Path

import dotenv
from pydantic import BaseModel, Field, SecretStr
from pydantic_settings import BaseSettings, SettingsConfigDict

from aa_rag.gtypes.enums import (
    IndexType,
    DBMode,
    RetrieveType,
)

dotenv.load_dotenv(Path(".env").absolute())


def load_env(key: str, default=None):
    """
    Load environment variable from .env file. Convert to python object if possible.
    Args:
        key: Environment variable key.
        default: Default value if key not found.
    Returns:
        Python object.
    """

    value = os.getenv(key, default)
    try:
        return ast.literal_eval(value)
    except Exception:
        return value


class Server(BaseModel):
    host: str = Field(default="0.0.0.0", description="The host address for the server.")
    port: int = Field(
        default=222, description="The port number on which the server listens."
    )


class OpenAI(BaseModel):
    api_key: SecretStr = Field(
        default=load_env("OPENAI_API_KEY"),
        alias="OPENAI_API_KEY",
        description="API key for accessing OpenAI services.",
    )
    base_url: str = Field(
        default=load_env("OPENAI_BASE_URL", "https://api.openai.com/v1"),
        alias="OPENAI_BASE_URL",
        description="Base URL for OpenAI API requests.",
    )


class DB(BaseModel):
    class Vector(BaseModel):
        uri: str = Field(
            default="./db/lancedb", description="URI for the vector database location."
        )

        mode: DBMode = Field(
            default=DBMode.UPSERT, description="Mode of operation for the database."
        )

    class NoSQL(BaseModel):
        uri: str = Field(
            default="./db/db.json",
            description="URI for the relational database location.",
        )

        mode: DBMode = Field(
            default=DBMode.UPSERT, description="Mode of operation for the database."
        )

    vector: Vector = Field(
        default_factory=Vector, description="Configuration for the vector database."
    )

    nosql: NoSQL = Field(
        default_factory=NoSQL, description="Configuration for the document database."
    )


class Embedding(BaseModel):
    model: str = Field(
        default="text-embedding-3-small",
        description="Model used for generating text embeddings.",
    )


class LLM(BaseModel):
    model: str = Field(
        default="gpt-4o",
        description="Model used for generating text embeddings.",
    )


class Index(BaseModel):
    type: IndexType = Field(
        default=IndexType.CHUNK, description="Type of index used for data retrieval."
    )
    chunk_size: int = Field(
        default=load_env("INDEX_CHUNK_SIZE", 512),
        description="Size of each chunk in the index.",
    )
    overlap_size: int = Field(
        default=load_env("INDEX_OVERLAP_SIZE", 100),
        description="Overlap size between chunks in the index.",
    )


class Retrieve(BaseModel):
    class Weight(BaseModel):
        dense: float = Field(
            default=0.5, description="Weight for dense retrieval methods."
        )
        sparse: float = Field(
            default=0.5, description="Weight for sparse retrieval methods."
        )

    type: RetrieveType = Field(
        default=RetrieveType.HYBRID, description="Type of retrieval strategy used."
    )
    k: int = Field(default=3, description="Number of top results to retrieve.")
    weight: Weight = Field(
        default_factory=Weight, description="Weights for different retrieval methods."
    )
    only_page_content: bool = Field(
        default=load_env("ONLY_PAGE_CONTENT", False),
        alias="ONLY_PAGE_CONTENT",
        description="Flag to retrieve only page content.",
    )


class Settings(BaseSettings):
    server: Server = Field(
        default_factory=Server, description="Server configuration settings."
    )
    openai: OpenAI = Field(
        default_factory=OpenAI, description="OpenAI API configuration settings."
    )

    db: DB = Field(default_factory=DB, description="Database configuration settings.")
    embedding: Embedding = Field(
        default_factory=Embedding, description="Embedding model configuration settings."
    )
    index: Index = Field(
        default_factory=Index, description="Index configuration settings."
    )
    retrieve: Retrieve = Field(
        default_factory=Retrieve,
        description="Retrieval strategy configuration settings.",
    )

    llm: LLM = Field(
        default_factory=LLM,
        description="Language model configuration settings.",
    )

    # 这里禁用了自动的 CLI 解析
    model_config = SettingsConfigDict(
        env_file=".env",
        env_nested_delimiter="_",
        extra="ignore",
    )


setting = Settings()



"""
After running this script, the embedded database file will be automatically created in the default location (./db/sqlite.db).
At the same time, the csv file is parsed and written to the solution table.

"""
# !! This script does not have the ability to merge data in solution table

import json

import pandas as pd


def csv2db(csv_path: str, sqlite_db_path: str):
    source_data = pd.read_csv(csv_path)
    source_data = source_data[["name", "os", "arch", "procedure", "url"]]

    duplicated_data = source_data.drop_duplicates()
    duplicated_data["os"].fillna("Universal", inplace=True)
    duplicated_data["arch"].fillna("Universal", inplace=True)

    def f1(row):
        data = {"name": row["name"], "git_url": row["url"]}

        return json.dumps(data, ensure_ascii=False)

    def f2(row):
        data = [
            {
                "env_info": {"platform": row["os"], "arch": row["arch"]},
                "procedure": row["procedure"],
            }
        ]

        return json.dumps(data, ensure_ascii=False)

    duplicated_data["project_meta"] = duplicated_data.apply(f1, axis=1)
    duplicated_data["guides"] = duplicated_data.apply(f2, axis=1)
    duplicated_data.drop(
        columns=["os", "arch", "procedure", "url", "name"], inplace=True
    )

    import sqlite3

    conn = sqlite3.connect(sqlite_db_path)
    table_name = "solution"
    conn.execute(f"""
                CREATE TABLE IF NOT EXISTS {table_name} (
                    project_id INTEGER PRIMARY KEY AUTOINCREMENT,
                    guides TEXT NOT NULL,
                    project_meta TEXT NOT NULL)""")

    conn.commit()
    duplicated_data.to_sql("solution", conn, if_exists="append", index=False)
    conn.close()


if __name__ == "__main__":
    import argparse

    parser = argparse.ArgumentParser(
        description="csv from feishu to sqlite as solution knowledge base"
    )

    # 添加可选参数
    parser.add_argument("csvpath")

    parser.add_argument("--sqlite-db-path", default="./db/sqlite.db")

    # 解析参数
    args = parser.parse_args()
    csv2db(csv_path=args.csvpath, sqlite_db_path=args.sqlite_db_path)
